//------------------------------------------------------------------
#property copyright "www.forex-station.com"
#property link      "www.forex-station.com"
//------------------------------------------------------------------
#property indicator_chart_window
#property indicator_buffers 5
#property indicator_color1 clrYellow
#property indicator_color2 clrDeepPink
#property indicator_color3 clrDeepPink
#property indicator_width1 3
#property indicator_width2 3
#property indicator_width3 3
#property strict


enum enTimeFrames
{
   tf_cu,         // Current time frame
   tf_m1,         // 1 minute
   tf_m2,         // 2 minutes
   tf_m3,         // 3 minutes
   tf_m4,         // 4 minutes
   tf_m5,         // 5 minutes
   tf_m6,         // 6 minutes
   tf_m10 =10,    // 10 minutes
   tf_m12 =12,    // 12 minutes
   tf_m15 =15,    // 15 minutes
   tf_m20 =20,    // 20 minutes
   tf_m30 =30,    // 30 minutes
   tf_h1  =60,    // 1 hour
   tf_h2  =120,   // 2 hours
   tf_h3  =180,   // 3 hours
   tf_h4  =240,   // 4 hours
   tf_h6  =360,   // 6 hours
   tf_h8  =480,   // 8 hours
   tf_h12 =720,   // 12 hours
   tf_d1  =1440,  // daily
   tf_w1  =10080, // weekly
   tf_mn  =43200  // monthly
};
enum enPrices
{
   pr_close,      // Close
   pr_open,       // Open
   pr_high,       // High
   pr_low,        // Low
   pr_median,     // Median
   pr_typical,    // Typical
   pr_weighted,   // Weighted
   pr_average,    // Average (high+low+open+close)/4
   pr_medianb,    // Average median body (open+close)/2
   pr_tbiased,    // Trend biased price
   pr_tbiased2,   // Trend biased (extreme) price
   pr_haclose,    // Heiken ashi close
   pr_haopen ,    // Heiken ashi open
   pr_hahigh,     // Heiken ashi high
   pr_halow,      // Heiken ashi low
   pr_hamedian,   // Heiken ashi median
   pr_hatypical,  // Heiken ashi typical
   pr_haweighted, // Heiken ashi weighted
   pr_haaverage,  // Heiken ashi average
   pr_hamedianb,  // Heiken ashi median body
   pr_hatbiased,  // Heiken ashi trend biased price
   pr_hatbiased2, // Heiken ashi trend biased (extreme) price
   pr_habclose,   // Heiken ashi (better formula) close
   pr_habopen ,   // Heiken ashi (better formula) open
   pr_habhigh,    // Heiken ashi (better formula) high
   pr_hablow,     // Heiken ashi (better formula) low
   pr_habmedian,  // Heiken ashi (better formula) median
   pr_habtypical, // Heiken ashi (better formula) typical
   pr_habweighted,// Heiken ashi (better formula) weighted
   pr_habaverage, // Heiken ashi (better formula) average
   pr_habmedianb, // Heiken ashi (better formula) median body
   pr_habtbiased, // Heiken ashi (better formula) trend biased price
   pr_habtbiased2 // Heiken ashi (better formula) trend biased (extreme) price
};

extern enTimeFrames     TimeFrame       = tf_cu;                // Time frame to use
input enPrices          Price           = pr_close;             // Price
input int               ArrowCodeUp     = 159;                  // Arrow code up
input int               ArrowCodeDn     = 159;                  // Arrow code down
input double            ArrowGapUp      = 0.5;                  // Gap for arrow up        
input double            ArrowGapDn      = 0.5;                  // Gap for arrow down
input color             ArrowUpColor    = clrLimeGreen;         // Up arrow color
input color             ArrowDnColor    = clrDeepPink;          // Down arrow color                    
input int               ArrowUpSize     = 2;                    // Up arrow size
input int               ArrowDnSize     = 2;                    // Down arrow size

double ma[],maDa[],maDb[],arrowu[],arrowd[],slope[];
int timeFrame;

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int OnInit()
{
   IndicatorBuffers(6);
   SetIndexBuffer(0,ma,    INDICATOR_DATA);
   SetIndexBuffer(1,maDa,  INDICATOR_DATA);
   SetIndexBuffer(2,maDb,  INDICATOR_DATA);
   SetIndexBuffer(3,arrowu,INDICATOR_DATA); SetIndexStyle(3,DRAW_ARROW,EMPTY,ArrowUpSize,ArrowUpColor); SetIndexArrow(3,ArrowCodeUp);
   SetIndexBuffer(4,arrowd,INDICATOR_DATA); SetIndexStyle(4,DRAW_ARROW,EMPTY,ArrowDnSize,ArrowDnColor); SetIndexArrow(4,ArrowCodeDn);
   SetIndexBuffer(5,slope);
   timeFrame = MathMax(TimeFrame,_Period);
         if (MathFloor(timeFrame/_Period)*_Period != timeFrame) timeFrame = _Period;
   return(INIT_SUCCEEDED);
}
void OnDeinit(const int reason) { Comment(" ");}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double   &open[],
                const double   &high[],
                const double   &low[],
                const double   &close[],
                const long     &tick_volume[],
                const long     &volume[],
                const int      &spread[])
{
   int counted_bars = prev_calculated;
      if(counted_bars < 0) return(-1);
      if(counted_bars > 0) counted_bars--;
           int limit=fmin(rates_total-counted_bars,rates_total-1);

   //
   //
   //
   //
   //
      
   if (slope[limit]==-1) CleanPoint(limit,maDa,maDb);
   for(int i = limit; i >= 0; i--)
   {  
      datetime startingTime;
      int      barsPassed,startBar=i;
         while (true)
         {
            if (timeFrame<60)
            {
               startingTime   = StrToTime(TimeToStr(time[i],TIME_DATE)+toHour(TimeHour(time[i])));
               barsPassed     = (int)floor((time[i]-startingTime)/(timeFrame*60));
               startBar       = iBarShift(NULL,0,startingTime+barsPassed*timeFrame*60);
               break;
            }
            if (timeFrame<1440)
            {
               startingTime   = StrToTime(TimeToStr(time[i],TIME_DATE)+" 00:00");
               barsPassed     = (int)floor((time[i]-startingTime)/(timeFrame*60));
               startBar       = iBarShift(NULL,0,startingTime+barsPassed*timeFrame*60);
               break;
            }
            startingTime = iTime(NULL,timeFrame,iBarShift(NULL,timeFrame,time[i])); if (timeFrame==tf_w1) startingTime+=1440*60;
            startBar     = iBarShift(NULL,0,startingTime);
            break;
         }         
         Comment(startBar-i+1);
         ma[i]     = iLwma(getPrice(Price,open,close,high,low,i,rates_total),startBar-i+1,i,rates_total);
         slope[i]  = (i<rates_total-1) ? (ma[i]>ma[i+1]&& Close[i+1]>Open[i+1] ) ? 1 : (ma[i]<ma[i+1]&& Close[i+1]<Open[i+1]) ? -1 : slope[i+1] : 0;            
         maDa[i]   = maDb[i]   = EMPTY_VALUE;   if (slope[i]==-1) PlotPoint(i,maDa,maDb,ma);
         arrowu[i] = arrowd[i] = EMPTY_VALUE;
         if (i<rates_total-1 && slope[i]!= slope[i+1])
         {
           if (slope[i] ==  1) arrowu[i] = fmin(ma[i],low[i] )-iATR(NULL,0,15,i)*ArrowGapUp;
           if (slope[i] == -1) arrowd[i] = fmax(ma[i],high[i])+iATR(NULL,0,15,i)*ArrowGapDn;
         }    
   }
return(rates_total);
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

double workLwma[][1];
double iLwma(double price, int period, int r, int bars, int instanceNo=0)
{
   if (ArrayRange(workLwma,0)!= bars) ArrayResize(workLwma,bars); r=bars-r-1;
   
   //
   //
   //
   //
   //
   
   workLwma[r][instanceNo] = price;
      double sumw = period;
      double sum  = period*price;
      for(int k=1; k<period && (r-k)>=0; k++)
      {
         double weight = period-k;
                sumw  += weight;
                sum   += weight*workLwma[r-k][instanceNo];  
      }             
      return(sum/sumw);
}

//-------------------------------------------------------------------
//                                                                  
//-------------------------------------------------------------------
//
//
//
//
//

void CleanPoint(int i,double& first[],double& second[])
{
   if (i>=Bars-3) return;
   if ((second[i]  != EMPTY_VALUE) && (second[i+1] != EMPTY_VALUE))
        second[i+1] = EMPTY_VALUE;
   else
      if ((first[i] != EMPTY_VALUE) && (first[i+1] != EMPTY_VALUE) && (first[i+2] == EMPTY_VALUE))
          first[i+1] = EMPTY_VALUE;
}

void PlotPoint(int i,double& first[],double& second[],double& from[])
{
   if (i>=Bars-2) return;
   if (first[i+1] == EMPTY_VALUE)
      if (first[i+2] == EMPTY_VALUE) 
            { first[i]  = from[i];  first[i+1]  = from[i+1]; second[i] = EMPTY_VALUE; }
      else  { second[i] =  from[i]; second[i+1] = from[i+1]; first[i]  = EMPTY_VALUE; }
   else     { first[i]  = from[i];                           second[i] = EMPTY_VALUE; }
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

#define _prHABF(_prtype) (_prtype>=pr_habclose && _prtype<=pr_habtbiased2)
#define _priceInstances     1
#define _priceInstancesSize 4
double workHa[][_priceInstances*_priceInstancesSize];
double getPrice(int tprice, const double& open[], const double& close[], const double& high[], const double& low[], int i, int bars, int instanceNo=0)
{
     if (tprice>=pr_haclose)
     {
         if (ArrayRange(workHa,0)!= bars) ArrayResize(workHa,bars); instanceNo*=_priceInstancesSize; int r = bars-i-1;
         
         //
         //
         //
         //
         //
         
         double haOpen  = (r>0) ? (open[i+1]+close[i+1])*0.5 : (open[i]+close[i])*0.5;
         double haClose = (open[i]+high[i]+low[i]+close[i])*0.25;
         if (_prHABF(tprice))
               if (high[i]!=low[i])
                     haClose = (open[i]+close[i])/2.0+(((close[i]-open[i])/(high[i]-low[i]))*fabs((close[i]-open[i])/2.0));
               else  haClose = (open[i]+close[i])/2.0; 
         double haHigh  = fmax(high[i],fmax(haOpen,haClose));
         double haLow   = fmin(low[i] ,fmin(haOpen,haClose));

         //
         //
         //
         //
         //
         
         if(haOpen<haClose) { workHa[r][instanceNo+0] = haLow;  workHa[r][instanceNo+1] = haHigh; } 
         else               { workHa[r][instanceNo+0] = haHigh; workHa[r][instanceNo+1] = haLow;  } 
                              workHa[r][instanceNo+2] = haOpen;
                              workHa[r][instanceNo+3] = haClose;
         //
         //
         //
         //
         //
         
         switch (tprice)
         {
            case pr_haclose:
            case pr_habclose:    return(haClose);
            case pr_haopen:   
            case pr_habopen:     return(haOpen);
            case pr_hahigh: 
            case pr_habhigh:     return(haHigh);
            case pr_halow:    
            case pr_hablow:      return(haLow);
            case pr_hamedian:
            case pr_habmedian:   return((haHigh+haLow)/2.0);
            case pr_hamedianb:
            case pr_habmedianb:  return((haOpen+haClose)/2.0);
            case pr_hatypical:
            case pr_habtypical:  return((haHigh+haLow+haClose)/3.0);
            case pr_haweighted:
            case pr_habweighted: return((haHigh+haLow+haClose+haClose)/4.0);
            case pr_haaverage:  
            case pr_habaverage:  return((haHigh+haLow+haClose+haOpen)/4.0);
            case pr_hatbiased:
            case pr_habtbiased:
               if (haClose>haOpen)
                     return((haHigh+haClose)/2.0);
               else  return((haLow+haClose)/2.0);        
            case pr_hatbiased2:
            case pr_habtbiased2:
               if (haClose>haOpen)  return(haHigh);
               if (haClose<haOpen)  return(haLow);
                                    return(haClose);        
         }
   }
   
   //
   //
   //
   //
   //
   
   switch (tprice)
   {
      case pr_close:     return(close[i]);
      case pr_open:      return(open[i]);
      case pr_high:      return(high[i]);
      case pr_low:       return(low[i]);
      case pr_median:    return((high[i]+low[i])/2.0);
      case pr_medianb:   return((open[i]+close[i])/2.0);
      case pr_typical:   return((high[i]+low[i]+close[i])/3.0);
      case pr_weighted:  return((high[i]+low[i]+close[i]+close[i])/4.0);
      case pr_average:   return((high[i]+low[i]+close[i]+open[i])/4.0);
      case pr_tbiased:   
               if (close[i]>open[i])
                     return((high[i]+close[i])/2.0);
               else  return((low[i]+close[i])/2.0);        
      case pr_tbiased2:   
               if (close[i]>open[i]) return(high[i]);
               if (close[i]<open[i]) return(low[i]);
                                     return(close[i]);        
   }
   return(0);
}

//
//
//
//
//

string toHour(int hour)
{
   if (hour<10)
         return(" 0"+(string)hour+":00");
   else  return(" " +(string)hour+":00");
}